@@ -93,6 +93,10 @@
 #ifndef _UB_UNBOUND_H
 #define _UB_UNBOUND_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * The validation context is created to hold the resolver status,
  * validation keys and a small cache (containing messages, rrsets,
@@ -181,6 +185,14 @@ struct ub_result {
 	 * This means the data is from a domain where data is not signed.
 	 */
 	int bogus;
+	
+	/**
+	 * If the result is bogus this contains a string (zero terminated)
+	 * that describes the failure.  There may be other errors as well
+	 * as the one described, the description may not be perfectly accurate.
+	 * Is NULL if the result is not bogus.
+	 */
+	char* why_bogus;
 };
 
 /**
@@ -219,7 +231,7 @@ void ub_ctx_delete(struct ub_ctx* ctx);
  * @param ctx: context.
  * @param opt: option name from the unbound.conf config file format.
  *	(not all settings applicable). The name includes the trailing ':'
- *	for example ub_ctx_set_option("logfile:", "mylog.txt");
+ *	for example ub_ctx_set_option(ctx, "logfile:", "mylog.txt");
  * 	This is a power-users interface that lets you specify all sorts
  * 	of options.
  * 	For some specific options, such as adding trust anchors, special
@@ -229,6 +241,22 @@ void ub_ctx_delete(struct ub_ctx* ctx);
  */
 int ub_ctx_set_option(struct ub_ctx* ctx, char* opt, char* val);
 
+/**
+ * Get an option from the context.
+ * @param ctx: context.
+ * @param opt: option name from the unbound.conf config file format.
+ *	(not all settings applicable). The name excludes the trailing ':'
+ *	for example ub_ctx_get_option(ctx, "logfile", &result);
+ * 	This is a power-users interface that lets you specify all sorts
+ * 	of options.
+ * @param str: the string is malloced and returned here. NULL on error.
+ * 	The caller must free() the string.  In cases with multiple 
+ * 	entries (auto-trust-anchor-file), a newline delimited list is 
+ * 	returned in the string.
+ * @return 0 if OK else an error code (malloc failure, syntax error).
+ */
+int ub_ctx_get_option(struct ub_ctx* ctx, char* opt, char** str);
+
 /**
  * setup configuration for the given context.
  * @param ctx: context.
@@ -515,4 +543,14 @@ int ub_ctx_data_add(struct ub_ctx* ctx, char *data);
  */
 int ub_ctx_data_remove(struct ub_ctx* ctx, char *data);
 
+/**
+ * Get a version string from the libunbound implementation.
+ * @return a static constant string with the version number.
+ */
+const char* ub_version(void);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* _UB_UNBOUND_H */
